/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 *
 *     This file is part of Tav4SB.
 *
 *     Tav4SB is free software: you can redistribute it and/or modify it under
 *     the terms of the GNU Affero General Public License as published by the
 *     Free Software Foundation, either version 3 of the License, or (at your
 *     option) any later version.
 *
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 *
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.workflows.bsh;

import java.math.BigDecimal;
import java.math.MathContext;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class CalculateThreshold {

    private static BigDecimal arithmeticMean(List<BigDecimal> nums) {
        BigDecimal ret = new BigDecimal(0);
        for (BigDecimal num : nums)
            ret = ret.add(num);
        return ret.divide(new BigDecimal(nums.size()), MathContext.DECIMAL128);
    }

    private static BigDecimal median(List<BigDecimal> nums) {
        // naive approach: reduction to sort
        List<BigDecimal> temp = new ArrayList<BigDecimal>(nums);
        Collections.sort(temp);
        int half = temp.size() / 2;
        return temp.size() % 2 == 1 ? temp.get(half) : arithmeticMean(temp.subList(half-1, half+1));
    }

    private static String MEAN_METHOD = "mean";
    private static String MEDIAN_METHOD = "median";

    public static void main(String[] args) {
        List<String> numbers = Arrays.asList(new String[] {
                "8", "16", "32", "1", "2", "4"
        });
        String method = null; // "mean" or "median" (default)

        List<BigDecimal> numList = new ArrayList<BigDecimal>(numbers.size());
        for (String numStr : numbers)
            numList.add(new BigDecimal(numStr));
        if (method == null)
            method = MEDIAN_METHOD;

        BigDecimal threshold;
        if (method.equals(MEAN_METHOD))
            threshold = arithmeticMean(numList);
        else if (method.equals(MEDIAN_METHOD))
            threshold = median(numList);
        else
            throw new RuntimeException("Unsupported threshold method \"" + method + "\".");

        System.out.println(threshold);
    }

}
