/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.workflows.bsh;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class PointListXmlToValuesListList {

    static final String NAME_ATTRNAME = "name";

    private static Map<String, String> pointMap(Element pEl) {
        NodeList coordinateNodes = pEl.getChildNodes();
        Map<String, String> coordMap = new HashMap<String, String>(coordinateNodes.getLength());
        for (int i = 0; i < coordinateNodes.getLength(); i++) {
            Node coordinateNode = coordinateNodes.item(i);
            if (coordinateNode.getNodeType() == Node.ELEMENT_NODE) {
                Element coordinateEl = (Element) coordinateNode;
                String name = coordinateEl.getAttributes().getNamedItem(NAME_ATTRNAME).getNodeValue();
                if (!coordMap.containsKey(name)) { // ignore duplicate coordinates
                    coordMap.put(name, coordinateEl.getTextContent());
                }
            }
        }
        return coordMap;
    }

    private static List<String> filterCoordinates(
            Map<String, String> point, List<String> keys) {
        List<String> ret = new ArrayList<String>(keys.size());
        for (String key: keys)
            ret.add(point.containsKey(key) ? point.get(key) : "NA");
        return ret;
    }

    public static void main(String[] args) throws SAXException, IOException, ParserConfigurationException {
//        String pointListXml = ""+
//        "<timepoints>     <timepoint>         <time name=\"time\">0</time>         <variable name=\"m1\">12</variable>         <variable name=\"m2\">10</variable>         <variable name=\"m3\">0</variable>         <variable name=\"m4\">0</variable>         <variable name=\"cell\">1</variable>     </timepoint>     <timepoint>         <time name=\"time\">3</time>         <variable name=\"m1\">1.53796</variable>         <variable name=\"m2\">3.3976</variable>         <variable name=\"m3\">6.6024</variable>         <variable name=\"m4\">3.85964</variable>         <variable name=\"cell\">1</variable>     </timepoint>     <timepoint>         <time name=\"time\">6</time>         <variable name=\"m1\">0.216087</variable>         <variable name=\"m2\">5.59542</variable>         <variable name=\"m3\">4.40458</variable>         <variable name=\"m4\">7.37933</variable>         <variable name=\"cell\">1</variable>     </timepoint>     <timepoint>         <time name=\"time\">9</time>         <variable name=\"m1\">0.0395318</variable>         <variable name=\"m2\">7.54475</variable>         <variable name=\"m3\">2.45525</variable>         <variable name=\"m4\">9.50522</variable>         <variable name=\"cell\">1</variable>     </timepoint>     <timepoint>         <time name=\"time\">12</time>         <variable name=\"m1\">0.0157706</variable>         <variable name=\"m2\">8.67987</variable>         <variable name=\"m3\">1.32013</variable>         <variable name=\"m4\">10.6641</variable>         <variable name=\"cell\">1</variable>     </timepoint>     <timepoint>         <time name=\"time\">15</time>         <variable name=\"m1\">0.00764254</variable>         <variable name=\"m2\">9.29325</variable>         <variable name=\"m3\">0.70675</variable>         <variable name=\"m4\">11.2856</variable>         <variable name=\"cell\">1</variable>     </timepoint>     <timepoint>         <time name=\"time\">18</time>         <variable name=\"m1\">0.00389938</variable>         <variable name=\"m2\">9.62205</variable>         <variable name=\"m3\">0.377952</variable>         <variable name=\"m4\">11.6181</variable>         <variable name=\"cell\">1</variable>     </timepoint>     <timepoint>         <time name=\"time\">21</time>         <variable name=\"m1\">0.00203507</variable>         <variable name=\"m2\">9.79798</variable>         <variable name=\"m3\">0.202024</variable>         <variable name=\"m4\">11.7959</variable>         <variable name=\"cell\">1</variable>     </timepoint>     <timepoint>         <time name=\"time\">24</time>         <variable name=\"m1\">0.00107407</variable>         <variable name=\"m2\">9.89204</variable>         <variable name=\"m3\">0.107962</variable>         <variable name=\"m4\">11.891</variable>         <variable name=\"cell\">1</variable>     </timepoint>     <timepoint>         <time name=\"time\">27</time>         <variable name=\"m1\">0.000570154</variable>         <variable name=\"m2\">9.94231</variable>         <variable name=\"m3\">0.0576887</variable>         <variable name=\"m4\">11.9417</variable>         <variable name=\"cell\">1</variable>     </timepoint>     <timepoint>         <time name=\"time\">30</time>         <variable name=\"m1\">0.000303578</variable>         <variable name=\"m2\">9.96918</variable>         <variable name=\"m3\">0.0308237</variable>         <variable name=\"m4\">11.9689</variable>         <variable name=\"cell\">1</variable>     </timepoint> </timepoints>";
//        List<String> nameList = Arrays.asList(new String[] {"time"});
//        List<String> nameList = Arrays.asList(new String[] {"m4", "m3"});

        String pointListXml = ""+
        "<data>" +
            "<point>"+
            "    <coordinate name=\"v\">v1</coordinate>"+
            "    <coordinate name=\"x\">x1</coordinate>"+
            "    <coordinate name=\"y\">y1</coordinate>"+
            "    <coordinate name=\"y\">ERR</coordinate>"+
            "    <coordinate name=\"z\">z1</coordinate>"+
            "</point>"+
            "<point>"+
            "    <coordinate name=\"v\">v2</coordinate>"+
            "    <coordinate name=\"x\">x2</coordinate>"+
            "    <coordinate name=\"y\">y2</coordinate>"+
            "</point>"+
        "</data>";
        //List<String> nameList = Arrays.asList(new String[] {"x"});
        List<String> coordinateValueListList = Arrays.asList(new String[] {"z", "y"});


        NodeList pointNodeList = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new ByteArrayInputStream(pointListXml.getBytes())).getDocumentElement().getChildNodes();
        List<List<String>> coordinateValueList = new ArrayList<List<String>>(pointNodeList.getLength());
        for (int i = 0; i < pointNodeList.getLength(); i++) {
            Node pointNode = pointNodeList.item(i);
            if (pointNode.getNodeType() == Node.ELEMENT_NODE) {
                Map<String, String> point = pointMap((Element) pointNode);
                coordinateValueList.add(filterCoordinates(point, coordinateValueListList));
            }
        }

        System.out.println(Arrays.toString(coordinateValueList.toArray()));
    }
}
