/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.workflows.bsh;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSSerializer;
import org.xml.sax.SAXException;

public class ProjectCoordinates {

    static final String NAME_ATTRNAME = "name";

    static String doc2String(Document doc) {
        DOMImplementationLS domImplementation = (DOMImplementationLS) doc.getImplementation();
        LSSerializer lsSerializer = domImplementation.createLSSerializer();
        return lsSerializer.writeToString(doc);
    }

    public static void main(String[] args) throws ParserConfigurationException, SAXException, IOException, XPathExpressionException {
        String inXml = "" +
        "<data>" +
            "<point>" +
                "<coordinate name=\"x1\">1.1</coordinate>" +
                "<coordinate name=\"x2\">2.1</coordinate>" +
                "<coordinate name=\"x3\">3.1</coordinate>" +
                "<coordinate name=\"x4\">4.1</coordinate>" +
            "</point>" +
            "<point>" +
                "<coordinate name=\"x1\">1.2</coordinate>" +
                "<coordinate name=\"x2\">2.2</coordinate>" +
                "<coordinate name=\"x3\">3.2</coordinate>" +
                "<coordinate name=\"x4\">4.2</coordinate>" +
            "</point>" +
        "</data>";
        List<String> coordinateNameList = Arrays.asList(new String[]{"x3", "x1"});

        Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new ByteArrayInputStream(inXml.getBytes()));
        XPathExpression xpexpr = XPathFactory.newInstance().newXPath().compile("//coordinate");
        NodeList coordinateNodes = (NodeList) xpexpr.evaluate(doc, XPathConstants.NODESET);
        for (int i = 0; i < coordinateNodes.getLength(); i++) {
            Node coordinateNode = coordinateNodes.item(i);
            if (!coordinateNameList.contains(coordinateNode.getAttributes().getNamedItem(NAME_ATTRNAME).getNodeValue())) {
                coordinateNode.getParentNode().removeChild(coordinateNode);
            }
        }
        String outXml = doc2String(doc);

        System.out.print(outXml);
    }
}
