/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 *
 *     This file is part of Tav4SB.
 *
 *     Tav4SB is free software: you can redistribute it and/or modify it under
 *     the terms of the GNU Affero General Public License as published by the
 *     Free Software Foundation, either version 3 of the License, or (at your
 *     option) any later version.
 *
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 *
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/

package pl.edu.mimuw.bioputer.tav4sb.workflows.bsh;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

class DistributionObj {
    String name;
    List<MathType> parameters;
}

/**
 * @author trybik
 *
 */
public class SetDistributionXml {

    private static DistributionObj distribution(String n, List<String> pl) {

        String name = n;
        List<MathType> parameters = new ArrayList<MathType>(pl.size());
        for (String pStr : pl) {
            parameters.add(LibMathType.mathtype(pStr));
        }

        DistributionObj ret = new DistributionObj();
        ret.name = name;
        ret.parameters = parameters;
        return ret;
    }

    static String DISTRIBUTION_TAGNAME = "distribution";
    static String NAME_TAGNAME = "name";
    static String PARAMETER_TAGNAME = "parameter";

    private static String toTag(DistributionObj dist) throws XMLStreamException, FactoryConfigurationError {
        StringWriter sw = new StringWriter();
        XMLStreamWriter xw = XMLOutputFactory.newInstance().createXMLStreamWriter(sw);

        xw.writeStartDocument();
        xw.writeStartElement(DISTRIBUTION_TAGNAME);
        xw.writeNamespace(LibMathType.XSI_NS_PREFIX, LibMathType.XSI_NS_URI);
        xw.writeNamespace(LibMathType.MATH_NS_PREFIX, LibMathType.MATH_NS_URI);
        xw.writeStartElement(NAME_TAGNAME);
        xw.writeCharacters(dist.name);
        xw.writeEndElement();
        for (MathType mt : dist.parameters) {
            LibMathType.writeMathTypeElement(xw, mt, PARAMETER_TAGNAME);
        }
        xw.writeEndDocument();

        return sw.toString();
    }

    public static void main(String[] args) throws XMLStreamException, FactoryConfigurationError {
        String name = "normal";
        List<String> parameterList = Arrays.asList(new String[]{
                "10.0",
                "3.16228",
        });

        String distributionXml = toTag(distribution(name, parameterList));

        System.out.println(distributionXml);



        name = "uniform";
        parameterList = Arrays.asList(new String[]{
                "{ 0.0, 1.0 }",
        });

        distributionXml = toTag(distribution(name, parameterList));

        System.out.println(distributionXml);
    }

}
