/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 *
 *     This file is part of Tav4SB.
 *
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 *
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 *
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.workflows.bsh;

import java.io.StringWriter;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;



class OptionObj {
    String name;
    MathType math;
}

class ExceptionMessage extends RuntimeException {
    public ExceptionMessage(String msg) {
        super(msg);
    }
}

class OptionFormatException extends Exception {
    public OptionFormatException(String option, String msg) {
        super(option + (msg == null ? "": ("; " + msg)) + ".");
    }
}

public class SetOptionXmlList {

    private static String NAME_VALUE_SEP = "->";

    static OptionObj option(String optionstr) throws OptionFormatException {
        String[] nameAndValue = optionstr.split(NAME_VALUE_SEP, 2);
        String name = nameAndValue[0].trim();
        if (nameAndValue.length < 2) {
            throw new OptionFormatException(optionstr, "missing value");
        }
        MathType math = null;
        try {
            math = LibMathType.mathtype(nameAndValue[1]);
        } catch (ExceptionMessage e) {
            throw new OptionFormatException(optionstr, e.getMessage());
        }

        OptionObj ret = new OptionObj();
        ret.name = name;
        ret.math = math;
        return ret;
    }

    static String OPTION_TAGNAME = "option";
    static String NAME_TAGNAME = "name";
    static String VALUE_TAGNAME = "value";

    private static String toTag(OptionObj opt) throws XMLStreamException, FactoryConfigurationError {
        StringWriter sw = new StringWriter();
        XMLStreamWriter xw = XMLOutputFactory.newInstance().createXMLStreamWriter(sw);

        xw.writeStartDocument();
        xw.writeStartElement(OPTION_TAGNAME);
        xw.writeNamespace(LibMathType.XSI_NS_PREFIX, LibMathType.XSI_NS_URI);
        xw.writeNamespace(LibMathType.MATH_NS_PREFIX, LibMathType.MATH_NS_URI);
        xw.writeStartElement(NAME_TAGNAME);
        xw.writeCharacters(opt.name);
        xw.writeEndElement();
        LibMathType.writeMathTypeElement(xw, opt.math, VALUE_TAGNAME);
        xw.writeEndDocument();

        String ret = sw.toString();

        System.out.println(ret);
        return ret;
    }

    public static void main(String[] args) throws XMLStreamException, FactoryConfigurationError, OptionFormatException {
        List<String> input = Arrays.asList(new String[]{
                "AxesLabel ->{\"t [\\\"s\\\"]\", \"u(0)\", \"f(t, u(0),)\" }",
                "PlotLabel -> \"Function f of time t and initial condition u(0) for multiplier 1/2, 1, and 2.\"",
                "PlotLegend-> None",
                "MarkerColor -> { {0.5, 0.5, 0.5}, Red, Gray }",
                "OpacityLevel -> { 0.2, 1., 0.2 }",
                "test -> { {.0,\"}\\}\",{0.}}, sym }",
        });

        List<String> output = new LinkedList<String>();
        if (!input.isEmpty()) {
            for (String optionstr: input) {
                output.add(toTag(option(optionstr)));
            }
        }
    }
}
