/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.consumer.task;

import pl.edu.mimuw.bioputer.tav4sb.utils.xml.XmlTranslator;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.ArgsResultsTranslator;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.ObjectSysCmd;

/**
 * A AbstracWorker implementation that wraps a ObjectSysCmd<ArgsT, ResultsT>
 * command.
 *
 * @author trybik
 */
public class SysCmdWorker<XmlParamsT, ArgsT, ResultsT, XmlResultsT>
        extends AbstractWorker<XmlParamsT, XmlResultsT> {

    private final ObjectSysCmd<ArgsT, ResultsT> cmd;
    private final ArgsResultsTranslator<XmlParamsT, ArgsT, ResultsT, XmlResultsT> arTranslator;

    public SysCmdWorker(XmlTranslator<XmlResultsT, XmlParamsT> xmlTranslator,
            ObjectSysCmd<ArgsT, ResultsT> cmd,
            ArgsResultsTranslator<XmlParamsT, ArgsT, ResultsT, XmlResultsT> arTranslator) {
        super(xmlTranslator);
        this.cmd = cmd;
        this.arTranslator = arTranslator;
    }

    protected ObjectSysCmd<ArgsT, ResultsT> getCmd() {
        return cmd;
    }

    protected ArgsResultsTranslator<XmlParamsT, ArgsT, ResultsT, XmlResultsT> getArTranslator() {
        return arTranslator;
    }

    @Override
    public XmlResultsT doExecute(XmlParamsT param) throws Exception {
        return getArTranslator().toOuter(getCmd().run(getArTranslator().toInner(param)));
    }

}
