/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine;

import com.wolfram.jlink.MathLinkException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import org.apache.log4j.Logger;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.utils.MathematicaProperties;

/**
 *
 * @author michal
 */
public class MEngineManager {

    private static final MathematicaProperties props = new MathematicaProperties();
    static final Set<MEngine> availableEngines = new HashSet<MEngine>();
    static final Set<MEngine> busyEngines = new HashSet<MEngine>();
    static final Object enginesLock = new Object();
    static final Logger logger = Logger.getLogger(MEngineManager.class);

    static public void initialize() {
        int n = props.getMLinkInstances();
//        int n = props.getIntProperty("MLINK_INSTANCES");

        logger.debug("create " + n + " kernels");

        synchronized (enginesLock) {

            for (int i = 0; i < n; i++) {
                logger.debug("kernel " + i);
                try {
                    availableEngines.add(new MEngine());
                } catch (MathLinkException e) {
                    logger.error("exception during creation MEngine (" + e.getMessage() + ")");
                    logger.error(e);
                    e.printStackTrace();
                }
            }
        }

        logger.debug("started");
    }

    static public void shutdown() {

        logger.debug("begin");

        synchronized (enginesLock) {
            for (MEngine e : availableEngines) {
                e.close();
            }
            for (MEngine e : busyEngines) {
                e.close();
            }
        }

        logger.debug("end");
    }

    static public MEngine getEngine() throws MathLinkException {
        logger.debug("begin");
        MEngine engine;
        synchronized (enginesLock) {
            Iterator<MEngine> iterator = availableEngines.iterator();
            if (iterator.hasNext()) {
                engine = iterator.next();
                logger.debug("Changing engine #" + engine.getId() + " state to BUSY");
                availableEngines.remove(engine);
            } else {
                logger.debug("No engine available, creating new one");
                engine = new MEngine();
            }
            busyEngines.add(engine);
        }
        logger.debug("end");
        return engine;
    }

    static public void releaseEngine(MEngine engine) {
        logger.debug("begin");
        synchronized (enginesLock) {
            logger.debug("Changing engine #" + engine.getId() + " state to AVAILABLE");
            availableEngines.add(engine);
            busyEngines.remove(engine);
        }
        logger.debug("end");
    }
}
