/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker;

import com.wolfram.jlink.ExprFormatException;
import com.wolfram.jlink.MathLinkException;

import java.util.LinkedList;
import java.util.List;

import pl.edu.mimuw.bioputer.tav4sb.consumer.task.AbstractWorker;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineException;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.proc.CalcSensitivity;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.proc.plot.Plot;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.ArrayOfParamRanking;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.ArrayOfPlot;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.CalcSensitivityParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.CalcSensitivityResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.ParamRanking;
import pl.edu.mimuw.bioputer.tav4sb.utils.xml.XmlTranslator;

/**
 *
 * @author michal
 */
public class CalcSensitivityWorker extends AbstractWorker<CalcSensitivityParams, CalcSensitivityResult> {

    public CalcSensitivityWorker(XmlTranslator<CalcSensitivityResult, CalcSensitivityParams> translator) {
        super(translator);
    }

    @Override
    public CalcSensitivityResult doExecute(CalcSensitivityParams param) throws MathLinkException, MEngineException, ExprFormatException {
        List<Plot> plots = new LinkedList<Plot>();
        List<ParamRanking> paramRank = CalcSensitivity.calcSensitivity(
                param.getSelectedParams().getParamList(),
                param.getAcceptableSamples().getSampleList().getSample(),
                param.getUnacceptableSamples().getSampleList().getSample(),
                param.isGeneratePlots().booleanValue(), plots
        );
        ArrayOfParamRanking aopr = new ArrayOfParamRanking();
        aopr.getRanking().addAll(paramRank);
        ArrayOfPlot aop = new ArrayOfPlot();
        for (Plot p : plots) {
            aop.getPlot().add(p.getData());
        }
        CalcSensitivityResult result = new CalcSensitivityResult();
        result.setRankingList(aopr);
        result.setEcdfPlotList(aop);
        return result;
    }
}
