/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 *
 *     This file is part of Tav4SB.
 *
 *     Tav4SB is free software: you can redistribute it and/or modify it under
 *     the terms of the GNU Affero General Public License as published by the
 *     Free Software Foundation, either version 3 of the License, or (at your
 *     option) any later version.
 *
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 *
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker;

import com.wolfram.jlink.MathLinkException;

import pl.edu.mimuw.bioputer.tav4sb.consumer.task.AbstractWorker;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineException;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.proc.SampleDistribution;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.utils.XsdUtils;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.SampleDistributionParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.SampleDistributionResult;
import pl.edu.mimuw.bioputer.tav4sb.utils.xml.XmlTranslator;

/**
 * @author trybik
 *
 */
public class SampleDistributionWorker extends
        AbstractWorker<SampleDistributionParams, SampleDistributionResult> {

    public SampleDistributionWorker(
            XmlTranslator<SampleDistributionResult, SampleDistributionParams> translator) {
        super(translator);
    }

    @Override
    public SampleDistributionResult doExecute(SampleDistributionParams param)
            throws MathLinkException, MEngineException {


        double[][] samples = SampleDistribution.sampleDistribution(
                param.getNumber(),
                param.getDistribution(),
                param.getMethod()
        );
        SampleDistributionResult ret = new SampleDistributionResult();
        for (int i = 0; i < samples.length; i++) {
            ret.getSample().add(XsdUtils.doubleArray2NumericArray(samples[i]));
        }
        return ret;
    }
}
