(*******************************************************************************
*Copyright (C) 2011 Mikołaj Rybiński
*
*    This file is part of Tav4SB.
*
*    Tav4SB is free software: you can redistribute it and/or modify it under the
*    terms of the GNU Affero General Public License as published by the Free
*    Software Foundation, either version 3 of the License, or (at your option)
*    any later version.
*
*    Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
*    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
*    FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
*    more details.
*
*    You should have received a copy of the GNU Affero General Public License
*    along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
*
*    Contributors:
*        Paweł Banasik - initial implementation
*        Michał Lula - grid implementation
*        Mikołaj Rybiński - implementation and maintenance
*******************************************************************************)
Needs["PlotLegends`"]
Needs["Utilz`"]

(* Required opts:

Generic:
 ? title
 ? legend size, position, text space

Fun specific:
 ? col: lines and surfaces colors
 ? axes labels
 ? axes labels boldness
 ? data series names: in legend
 ? line types (dashed, dotted etc.)
 ? line width (thin, thick etc.)
 ? line/surface opacity
*)

Options[MathPlot] = {
    (* Convenience (PlotStyle) *)
    MarkerColor -> Automatic,
    ThicknessLevel -> 5, (* 1000 times parameter value for Thickness function *)
    OpacityLevel -> 1, (* parameter for Opacity function *)
    DashingType -> Infinity, (* either pre-defined: Dashed, Dotted, DotDashed
                                or parameter for the Dashing function
                              *)
    (* Plot *)
    (* labels *)
    AxesLabel -> None,
    PlotLabel -> None,
    LabelStyle -> Bold,
    TicksStyle -> Plain,
    (* misc *)
    ImageSize -> Large,
    PlotRange -> Full,
    ColorFunction -> None,

    (* Legend *)
    PlotLegend -> None, (* by default no legend *)
    LegendShadow -> None,
    LegendPosition -> {1.1, -0.25}(*,
    LegendSize -> {1, 0.5},
    LegendTextSpace -> 5*)
}



dashingOrId[par_Dashing] := par
dashingOrId[par_] := Dashing[par]

toRgb[col:_RGBColor] := col
toRgb[col:_GrayLevel|_CMYKColor|_Hue|_Blend|_Lighter|_Darker] := ColorConvert[col, "RGB"]
toRgb[col:{_?NumberQ,_?NumberQ,_?NumberQ}] :=
    RGBColor[col[[1]],col[[2]],col[[3]]]
appendPlotStyleColor[plotStyle_, Automatic] := plotStyle
appendPlotStyleColor[plotStyle_, color_] :=
    Append[#[[1]], toRgb[#[[2]]]] & /@ ZipLists[plotStyle, color]

setPlotStyle[allOpts_] :=
    Module[{ opacity, thickness, plotStyle },
        opacity = OpacityLevel /. allOpts;
        thickness = (ThicknessLevel /. allOpts) / 1000;
        dashing = DashingType /. allOpts;
        plotStyle = {Thickness[#[[1]]], Opacity[#[[2]]], dashingOrId[#[[3]]]} &
            /@ ZipLists[thickness, opacity, dashing];
        plotStyle = appendPlotStyleColor[plotStyle, MarkerColor /. allOpts];
        OptsUnion[{PlotStyle -> plotStyle}, allOpts]
    ]



mathPlot2DFilterOpts[None] := Options[ListPlot]
mathPlot2DFilterOpts[_] :=
    OptsUnion[mathPlot2DFilterOpts[None], Options[Legend], {PlotLegend -> None}]
dataSeriesPattern := {{_?NumberQ, _?NumberQ..}..}
sortDataSeries[dataSeries: dataSeriesPattern] := Sort[dataSeries]
sortDataSeries[dataSeriesList: {dataSeriesPattern..}] :=
    sortDataSeries /@ dataSeriesList

Options[MathPlot2D] = OptsUnion[{
    (* lines and markers *)
    Joined -> True,
    PlotMarkers -> {Automatic, Small}
}, Options[MathPlot]]
MathPlot2D[data_, opts___?OptionQ] :=
    Module[{ allOpts },
        allOpts = setPlotStyle[OptsDefaults[MathPlot2D, opts]];
        ListPlot[
            sortDataSeries[data],
            FilterOpts[allOpts, mathPlot2DFilterOpts[PlotLegend /. allOpts]]
        ]
    ]



countData3dSeries[series : {dataSeriesPattern..}] := Length[series]
countData3dSeries[series : dataSeriesPattern] := 1
genColors[Automatic, data_List] := nColors[countData3dSeries[data]]
genColors[colors_, _] := toRgb /@ colors
nColors[n_ ?PositiveIntegerQ] := ColorData[1] /@ Range[n]

Options[MathPlot3D] = OptsUnion[{
    (*OpacityLevel -> 0.5,*)
    (*PlotMarkers -> {Automatic, Medium}*)
    (* Plot options *)
    ColorFunction -> "TemperatureMap",
    Mesh -> None,
    LegendTextSpace -> 5,
    (* Plot3D native options *)
    ViewPoint -> {1, -2, 1.3} (* Symbolic: {Front, Top, Right} *)
}, Options[MathPlot]]
MathPlot3D[data_, opts___?OptionQ] :=
    Module[{ allOpts, colors, ptSize, plot },
        allOpts = OptsDefaults[MathPlot3D, opts];
        (* manually generate marker colors if requested *)
        colors = genColors[MarkerColor /. allOpts, data];
        allOpts = setPlotStyle[OptsUnion[{MarkerColor->colors}, allOpts]];
        (*TODO: parametrize pt size *)
        ptSize = PointSize[Medium];
        plot = Show[
            (* Plot surfaces *)
            ListPlot3D[
                sortDataSeries[data],
                FilterOpts[allOpts, Options[ListPlot3D]]
            ],
            (* Plot markers *)
            Graphics3D[{ptSize, #[[2]], Point[#[[1]]]}] & /@
                ZipLists[data, colors],
            FilterOpts[allOpts, Options[Show]]
        ];
        mathPlot3DLegend[plot, PlotLegend /. allOpts, colors, ptSize, allOpts]
    ]

mathPlot3DLegend[plot_, None, __] := plot
mathPlot3DLegend[plot_, seriesNames_, colors_, ptSize_, allOpts_] :=
    ShowLegend[
            plot,
            {   (* Plot legend *)
                { Graphics[{ptSize, #[[2]], Point[{0,0}]}], #[[1]] } & /@
                    ZipLists[seriesNames, colors],
                FilterOpts[allOpts, Options[Legend]]
            },
            FilterOpts[allOpts, Options[ShowLegend]]
    ]
