(*******************************************************************************
*Copyright (C) 2011 Mikołaj Rybiński
*
*    This file is part of Tav4SB.
*
*    Tav4SB is free software: you can redistribute it and/or modify it under the
*    terms of the GNU Affero General Public License as published by the Free
*    Software Foundation, either version 3 of the License, or (at your option)
*    any later version.
*
*    Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
*    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
*    FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
*    more details.
*
*    You should have received a copy of the GNU Affero General Public License
*    along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
*
*    Contributors:
*        Paweł Banasik - initial implementation
*        Michał Lula - grid implementation
*        Mikołaj Rybiński - implementation and maintenance
*******************************************************************************)
Needs["Utilz`"]

Options[SampleDist] = {
	Method -> "Random"
}

SampleDistribution[nr_, distSpecs:{{_String, _List}..}, opts___?OptionQ] :=
    Module[{allOpts, method, distVec, lhs, ret},
    	allOpts = OptsDefaults[SampleDist, opts];
    	method = ToLowerCase[Method /. allOpts];
    	distVec = (Distribution[#[[1]]] @@ #[[2]]) & /@ distSpecs;
    	ret = If[method=="random",
    	   RandomDistribution[#, nr] & /@ distVec
        ,
    	If[MemberQ[{"full", "lhs"}, method],
           lhs = LHS1D[#, nr] & /@ distVec;
           If[method=="lhs",
           	   RandomSample[#, nr] & /@ lhs,
           	   lhs
           ]
    	,
    	   (* TODO: do not fail silently *)
    	   ConstantArray[0, {Length[distSpecs], nr}]
    	]];
    	If[method=="full",
    	   Flatten[Outer @@ Prepend[ret, {##} &], Length[distSpecs]-1],
    	   Transpose[ret]
	    ]
    ]
Distribution[distName_String] := Module[{namePrefix},
        namePrefix = StringReplacePart[ToLowerCase[distName],
            StringTake[ToUpperCase[distName], 1], {1, 1}];
        Symbol[namePrefix <> "Distribution"]
    ]
RandomDistribution[dist_, nr_] := Module[{ret},
        (* Mathematica 8:
        RandomVariate[dist, nr]; *)
        (* Mathematica <8: *)
        ret = Quiet[RandomReal[dist, nr]];
        If[ListQ[ret], ret, RandomInteger[dist, nr]]
    ]
LHS1D[dist_, n_?PositiveIntegerQ] := InverseCDF[dist, #] & /@
    Flatten[RandomReal /@ (Table[{i, i + 1}, {i, 0, n - 1}]/n)]
