/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 *
 *     This file is part of Tav4SB.
 *
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 *
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 *
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker;

import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import pl.edu.mimuw.bioputer.tav4sb.consumer.utils.WorkersTest;

/**
 *
 * @author michal
 */
public class MathematicaWorkersTest
        extends WorkersTest
{

    public static String testMathPlot2D;
    public static String testMathPlot3D;
    public static String testSampleDistribution;
    public static String testCalcSensitivity;
    public static String testClassifySamples;
    public static String testPlotTrajectories;
    public static String testPlotTrajectories3D;
    public static String testSampleParams;
    public static String testSelectObservables;
    public static String testSelectThreshold;

    @BeforeClass
    public static void setUpClass() throws Exception {
        testMathPlot2D = getResourcePath("mathPlot2D");
        testMathPlot3D = getResourcePath("mathPlot3D");
        testSampleDistribution = getResourcePath("sampleDistribution");
        testCalcSensitivity = getResourcePath("calcSensitivity");
        testClassifySamples = getResourcePath("classifySamples");
        testPlotTrajectories = getResourcePath("plotTrajectories");
        testPlotTrajectories3D = getResourcePath("plotTrajectories3D");
        testSampleParams = getResourcePath("sampleParams");
        testSelectObservables = getResourcePath("selectObservables");
        testSelectThreshold = getResourcePath("selectThreshold");
    }

    @Test
    public void testMathPlot2D() {
        logger.debug("testMathPlot2D");
        test(MathPlotWorkerFactory.getInstance(), testMathPlot2D);
    }

    @Test
    public void testMathPlot3D() {
        logger.debug("testMathPlot3D");
        test(MathPlotWorkerFactory.getInstance(), testMathPlot3D);
    }

    @Test
    public void testSampleDistribution() {
        logger.debug("testSampleDistribution");
        test(SampleDistributionWorkerFactory.getInstance(), testSampleDistribution);
    }

    @Test
    public void testCalcSensitivity() {
        logger.debug("testCalcSensitivity");
        test(CalcSensitivityWorkerFactory.getInstance(), testCalcSensitivity);
    }

    @Test
    public void testClassifySamples() {
        logger.debug("testClassifySamples");
        test(ClassifySamplesWorkerFactory.getInstance(), testClassifySamples);
    }

    @Test
    public void testPlotTrajectories() {
        logger.debug("testPlotTrajectories");
        test(PlotTrajectoriesWorkerFactory.getInstance(), testPlotTrajectories);
    }

    //TODO: testPlotTrajectories3D unit test?
    @Ignore
    @Test
    public void testPlotTrajectories3D() {
        logger.debug("testPlotTrajectories3D");
        test(PlotTrajectories3DWorkerFactory.getInstance(), testPlotTrajectories3D);
    }

    @Test
    public void testSampleParams() {
        logger.debug("testSampleParams");
        test(SampleParamsWorkerFactory.getInstance(), testSampleParams);
    }

    @Test
    public void testSelectObservables() {
        logger.debug("testSelectObservables");
        test(SelectObservablesWorkerFactory.getInstance(), testSelectObservables);
    }

    @Test
    public void testSelectThreshold() {
        logger.debug("testSelectThreshold");
        test(SelectThresholdWorkerFactory.getInstance(), testSelectThreshold);
    }
}
