/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.odesolver;

import org.apache.log4j.Logger;
import pl.edu.mimuw.bioputer.tav4sb.consumer.task.TaskConsumerManager;
import pl.edu.mimuw.bioputer.tav4sb.logic.odesolver.utils.OdeSolverProperties;
import pl.edu.mimuw.bioputer.tav4sb.logic.odesolver.worker.OdeSolverWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.CommandLineArgs;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.IncorrectArgumentFormatException;

/**
 *
 * @author michal
 */
public class Main {

    private static final Logger logger = Logger.getLogger(Main.class);
    private static final OdeSolverProperties props = new OdeSolverProperties();

    public static void main(String[] args) {
        try {
            CommandLineArgs command = new CommandLineArgs(args);
            String brokerURL = command.getArgValue(props.BROKER_URL, props.getBrokerUrl());
            String queueName = command.getArgValue(props.QUEUE_NAME, props.getQueueName());
            String odeSolverTask = command.getArgValue(props.RUN_TASK, props.getRunTask());
            String threadNumber = command.getArgValue(props.THREAD_NUMBER, props.getThreadNumber());
            String odeSolverCommand = command.getArgValue(props.COMMAND, props.getCommand());
            if (command.isHelpSet()) {
                command.printHelp(System.out);
                return;
            }
            logger.info("Starting ...");
            logger.info("Broker URL: " + brokerURL);
            logger.info("Queue name: " + queueName);
            logger.info("OdeSolver task: " + odeSolverTask);
            logger.info("Thread number: " + threadNumber);
            logger.info("OdeSolver command: " + odeSolverCommand);
            TaskConsumerManager consumer = new TaskConsumerManager(brokerURL, queueName);
            consumer.registerFactory(odeSolverTask, OdeSolverWorkerFactory.getInstance(odeSolverCommand));
            consumer.run(Integer.parseInt(threadNumber));
        } catch (IncorrectArgumentFormatException e) {
            logger.error("Incorrect argument format");
            System.exit(1);
        } catch (Exception e) {
            logger.error("Exception occured: " + e.getMessage(), e);
            System.exit(1);
        }
    }
}
