/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.odesolver.worker;

import pl.edu.mimuw.bioputer.tav4sb.consumer.task.AbstractWorker;
import pl.edu.mimuw.bioputer.tav4sb.logic.odesolver.OdeSolver;
import pl.edu.mimuw.bioputer.tav4sb.logic.odesolver.OdeSolverError;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.odesolver.OdeSolverParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.odesolver.OdeSolverResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.odesolver.Trajectories;
import pl.edu.mimuw.bioputer.tav4sb.utils.xml.XmlTranslator;

/**
 *
 * @author michal
 */
public class OdeSolverWorker extends AbstractWorker<OdeSolverParams, OdeSolverResult> {
    private final OdeSolver simulator;

    public OdeSolverWorker(XmlTranslator<OdeSolverResult, OdeSolverParams> translator, OdeSolver s) {
        super(translator);
        this.simulator = s;
    }

    @Override
    public OdeSolverResult doExecute(OdeSolverParams param) throws Exception {
        OdeSolverResult result = new OdeSolverResult();
        try {
            Trajectories t = getSimulator().run(
                    param.getModel(), param.getSimt(), param.getSimtp(),
                    param.getAtol(), param.getRtol(),
                    param.isSensitivityAnalysis()
            );
            result.setTrajectories(t);
        } catch (OdeSolverError e) {
            result.setError(e.getMessage());
        }
        return result;
    }

    /**
     * @return the simulator
     */
    public OdeSolver getSimulator() {
        return simulator;
    }
}
