/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pl.edu.mimuw.bioputer.tav4sb.logic.prism;

import org.apache.log4j.Logger;
import pl.edu.mimuw.bioputer.tav4sb.consumer.task.TaskConsumerManager;
import pl.edu.mimuw.bioputer.tav4sb.logic.prism.utils.PrismProperties;
import pl.edu.mimuw.bioputer.tav4sb.logic.prism.worker.PrismWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.logic.prism.worker.Sbml2PrismWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.CommandLineArgs;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.IncorrectArgumentFormatException;

/**
 *
 * @author michal
 */
public class Main {

    private static final Logger logger = Logger.getLogger(Main.class);
    public static final PrismProperties props = new PrismProperties();

    public static void main(String[] args) {
        try {
            logger.info("Starting ...");
            CommandLineArgs command = new CommandLineArgs(args);
            String brokerURL = command.getArgValue(props.BROKER_URL, props.getBrokerUrl());
            String queueName = command.getArgValue(props.QUEUE_NAME, props.getQueueName());
            String threadNumber = command.getArgValue(props.THREAD_NUMBER, props.getThreadNumber());
            String prismCommad = command.getArgValue(props.PRISM_COMMAND, props.getPrismCommand());
            String prismTask = command.getArgValue(props.PRISM_RUN_TASK, props.getPrismRunTask());
            String sbml2PrismCommad = command.getArgValue(props.SBML2PRISM_COMMAND, props.getSbml2PrismCommand());
            String sbml2PrismTask = command.getArgValue(props.SBML2PRISM_RUN_TASK, props.getSbml2PrismRunTask());
            String sbml2PrismClasspath = command.getArgValue(props.SBML2PRISM_CLASSPATH, props.getSbml2PrismClasspath());
            String sbml2PrismClassname = command.getArgValue(props.SBML2PRISM_CLASSNAME, props.getSbml2PrismClassname());
            if (command.isHelpSet()) {
                command.printHelp(System.out);
                return;
            }
            logger.info("Broker URL: " + brokerURL);
            logger.info("Queue name: " + queueName);
            logger.info("Thread number: " + threadNumber);
            logger.info("Prism task: " + prismTask);
            logger.info("Prism command: " + prismCommad);
            logger.info("Sbml2Prism task: " + sbml2PrismTask);
            logger.info("Sbml2Prism command: " + sbml2PrismCommad);
            logger.info("Sbml2Prism classpath: " + sbml2PrismClasspath);
            logger.info("Sbml2Prism classname: " + sbml2PrismClassname);
            TaskConsumerManager consumer = new TaskConsumerManager(brokerURL, queueName);
            consumer.registerFactory(prismTask, PrismWorkerFactory.getInstance(prismCommad));
            consumer.registerFactory(sbml2PrismTask, Sbml2PrismWorkerFactory.getInstance(sbml2PrismCommad, sbml2PrismClasspath, sbml2PrismClassname));
            consumer.run(Integer.parseInt(threadNumber));
        } catch (IncorrectArgumentFormatException e) {
            logger.error("Incorrect argument format");
            System.exit(1);
        } catch (Exception e) {
            logger.error("Exception occured: " + e.getMessage(), e);
            System.exit(1);
        }
    }
}
