/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.prism;

import pl.edu.mimuw.bioputer.tav4sb.utils.file.SaveableStringContent;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import org.apache.log4j.Logger;

/**
 * Class represents result of prism model checker command line execution
 * @author michal
 */
public class PrismResult extends SaveableStringContent implements PrismFile {

    private static final Logger logger = Logger.getLogger(PrismResult.class);
    private static final String modelCheckingToken = "Model checking";
    private static final String resulToken = "Result";
    private static final String errorToken = "Error";
    private static final String TOKEN_VALUE_SEPARATOR = ": ";
    private final String prismStdOut;
    private final Map<PrismConstant, BigDecimal> constants = new HashMap<PrismConstant, BigDecimal>();
    private Map<String, Double> result;
    private String error;

    public PrismResult(String prismStdOut, Map<PrismConstant, BigDecimal> constants) {
        this.prismStdOut = prismStdOut;
        this.constants.putAll(constants);
        parseStdOut();
    }

    @Override
    public String getContent() {
        return this.prismStdOut;
    }

    public List<PrismConstant> getConstants() {
        List<PrismConstant> res = new LinkedList<PrismConstant>();
        res.addAll(this.constants.keySet());
        return res;
    }

    private String tokenValue(String token) {
        return token.split(TOKEN_VALUE_SEPARATOR)[1];
    }

    private void parseStdOut() {
        if (result == null && error == null) {
            result = new HashMap<String, Double>();
            Scanner s = new Scanner(prismStdOut);
            String l;
            String name = "";
            String strVal = "";
            Double val; // Double instead of BigDecimal because of inifnity
            while (s.hasNextLine()) {
                l = s.nextLine();
                if (l.startsWith(modelCheckingToken)) {
                    name = tokenValue(l);
                } else if (l.startsWith(resulToken)) {
                    strVal = tokenValue(l);
                    //logger.debug("result: " + name + " =(Double) " + strVal);
                    val = new Double(strVal);
                    result.put(name, val);
                    logger.debug("result: " + name + " = " + val);
                } else if (l.startsWith(errorToken)) {
                    error = tokenValue(l);
                    logger.debug("error: " + error);
                }
            }
        }
    }

    public Map<String, Double> getResult() {
        return result;
    }

    public String getError() {
        return error;
    }

    public Map<PrismConstant, BigDecimal> getConstantsValue() {
        Map<PrismConstant, BigDecimal> res = new HashMap<PrismConstant, BigDecimal>();
        res.putAll(this.constants);
        return res;
    }
}
