/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.prism;

import java.io.IOException;
import java.math.BigInteger;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.JavaCmdGeneric;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.AbstractJavaCmdInOutTranslator;


/**
 *
 * @author trybik
 */
public class Sbml2Prism extends JavaCmdGeneric<Sbml2PrismArgs, PrismModel> {

    private final String classpath;
    private final String classname;

    private static class Sbml2PrismInOutTranslator extends AbstractJavaCmdInOutTranslator<Sbml2PrismArgs, PrismModel> {
        public PrismModel fromStdOut(String out) {
            return new PrismModel(out);
        }
    }

    public Sbml2Prism(String javaCmd, String classpath, String classname) {
        super(javaCmd, new Sbml2PrismInOutTranslator());
        this.classpath = classpath;
        this.classname = classname;
    }

    public String getClasspath() {
        return classpath;
    }

    public String getClassname() {
        return classname;
    }

//    @Override
//    public PrismModel run(Sbml2PrismArgs args) throws IOException, Sbml2PrismException {
//        if (getOut().isEmpty() && !getErr().isEmpty()) {
//            throw new Sbml2PrismException(getErr());
//        }
//        return super.run(args);
//    }

    // TODO: sbml model class; use libsbml?
    public PrismModel run(String sbmlModel, BigInteger maxAmmount) throws IOException, InterruptedException {
        return run(new Sbml2PrismArgs(classpath, classname, sbmlModel, maxAmmount));
    }
    public PrismModel run(String sbmlModel) throws IOException, InterruptedException {
        return run(sbmlModel, null);
    }
}
