/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.prism.worker;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import pl.edu.mimuw.bioputer.tav4sb.consumer.task.SysCmdWorker;
import pl.edu.mimuw.bioputer.tav4sb.logic.prism.Prism;
import pl.edu.mimuw.bioputer.tav4sb.logic.prism.PrismArgs;
import pl.edu.mimuw.bioputer.tav4sb.logic.prism.PrismConstant;
import pl.edu.mimuw.bioputer.tav4sb.logic.prism.PrismProperties;
import pl.edu.mimuw.bioputer.tav4sb.logic.prism.PrismModel;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism.Value;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism.FiniteValue;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism.ResultValue;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism.PrismParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism.PrismResult;
import pl.edu.mimuw.bioputer.tav4sb.utils.xml.XmlTranslator;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.ArgsResultsTranslator;

/**
 * Basically just an ArgsResultsTranslator wrapper.
 *
 * @author michal
 */
public class PrismWorker extends SysCmdWorker<PrismParams, PrismArgs, pl.edu.mimuw.bioputer.tav4sb.logic.prism.PrismResult, PrismResult> {

    private static final ArgsResultsTranslator<PrismParams, PrismArgs, pl.edu.mimuw.bioputer.tav4sb.logic.prism.PrismResult, PrismResult> arTranslator =
            new ArgsResultsTranslator<PrismParams, PrismArgs, pl.edu.mimuw.bioputer.tav4sb.logic.prism.PrismResult, PrismResult>() {

                public PrismArgs toInner(PrismParams prismParam) {
                    PrismModel model = new PrismModel(prismParam.getModel());
                    PrismProperties properties = new PrismProperties(prismParam.getProperties());
                    Map<PrismConstant, BigDecimal> constValue = new HashMap<PrismConstant, BigDecimal>();
                    for (FiniteValue v : prismParam.getConstant()) {
                        PrismConstant c = new PrismConstant(v.getName());
                        // !!!!!!!!!!!!!!!!  temporal fix !!!!!!!!!!!!!!!
                        properties.getConstants().add(c);
                        // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                        constValue.put(c, v.getValue());
                    }
                    return new PrismArgs(model, properties, constValue);
                }

                public PrismResult toOuter(pl.edu.mimuw.bioputer.tav4sb.logic.prism.PrismResult result) {
                    PrismResult simResult = new PrismResult();
                    ResultValue resVal = new ResultValue();
                    for (Entry<PrismConstant, BigDecimal> e : result.getConstantsValue().entrySet()) {
                        FiniteValue v = new FiniteValue();
                        v.setName(e.getKey().getLabel());
                        v.setValue(e.getValue());
                        resVal.getConstant().add(v);
                    }
                    for (Entry<String, Double> e : result.getResult().entrySet()) {
                        Value v = new Value();
                        v.setName(e.getKey());
                        v.setValue(e.getValue());
                        resVal.getProperty().add(v);
                    }
                    resVal.setError(result.getError());
                    simResult.getResult().add(resVal);
                    return simResult;
                }
            };

    PrismWorker(XmlTranslator<PrismResult, PrismParams> translator, Prism prism) {
        super(translator, prism, arTranslator);
    }
}
