/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.prism.worker;

import pl.edu.mimuw.bioputer.tav4sb.consumer.task.Worker;
import pl.edu.mimuw.bioputer.tav4sb.consumer.task.WorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.logic.prism.Prism;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism.PrismParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism.PrismResult;
import pl.edu.mimuw.bioputer.tav4sb.utils.xml.XmlTranslator;

/**
 *
 * @author michal
 */
public class PrismWorkerFactory implements WorkerFactory {

    public static final String context = "pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism";
    private static final XmlTranslator<PrismResult, PrismParams> translator =
            new XmlTranslator<PrismResult, PrismParams>(context);
    private final String command;

    private PrismWorkerFactory(String command) {
        this.command = command;
    }

    private Prism getPrism() {
        // new command obj each time as it's not thread-safe
        return new Prism(command);
    }

    public Worker getWorker() {
        return new PrismWorker(translator, getPrism());
    }

    public static final PrismWorkerFactory getInstance(String command) {
        return new PrismWorkerFactory(command);
    }
}
