#!/bin/bash
#-------------------------------------------------------------------------------
# Copyright (C) 2011 Mikołaj Rybiński
#
#     This file is part of Tav4SB.
#
#     Tav4SB is free software: you can redistribute it and/or modify it under
#     the terms of the GNU Affero General Public License as published by the
#     Free Software Foundation, either version 3 of the License, or (at your
#     option) any later version.
#
#     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
#     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#     more details.
#
#     You should have received a copy of the GNU Affero General Public License
#     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
#
#     Contributors:
#         Paweł Banasik - initial implementation
#         Michał Lula - grid implementation
#         Mikołaj Rybiński - implementation and maintenance
#-------------------------------------------------------------------------------

usage() {
    echo "Usage:"
    echo "    $0 worker_prefix ... workerN_prefix"
    exit 1
}
[[ $# -eq 0 ]] && usage

JAR_SUFIX="-jar-with-dependencies.jar"

worker_start() {
    local jar=$1

    if [[ !( -d ${TAV4SB_LOG_ROOT} ) || !( -w ${TAV4SB_LOG_ROOT} ) ]];  then
        echo "ERROR: cannot write to the log directory ${TAV4SB_LOG_ROOT}. Assert that the TAV4SB_LOG_ROOT environmental variable is set correctly."
        exit 1
    fi
    local log_file=${TAV4SB_LOG_ROOT}/$(basename $jar ${JAR_SUFIX}).out
    echo "Running: java -jar ${jar} >${log_file} 2>&1 &"
    java -jar ${jar} >$log_file 2>&1 &
}

worker_find() {
    local jar_prefix=$1

    local pattern="${jar_prefix}*${JAR_SUFIX}"
    local folder=${TAV4SB_PROJECT_ROOT}
    local jar=$(find ${folder} -iname ${pattern})
    if [ "${jar}" = "" ]; then
        echo "ERROR: couldn't find worker matching ${pattern} in ${folder}."
        exit 1
    fi
    JAR=${jar}
}

group_pid() {
    local pid=$1
    PGID=$(ps -p $pid -o pgid=)
}

kill_children() {
    group_pid $$
    trap 'exit 0' INT KILL TERM
    trap 'kill -9 -$(echo $PGID)' 0
}

# for register and any other unexpected cases
kill_children
for jar_prefix in "$@"; do
    worker_find $jar_prefix
    worker_start $JAR
done
wait

