/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pl.edu.mimuw.bioputer.tav4sb.register;

import java.io.IOException;
import org.apache.log4j.Logger;
import pl.edu.mimuw.bioputer.tav4sb.register.comp.PrismComputationFactory;
import pl.edu.mimuw.bioputer.tav4sb.register.Register;
import pl.edu.mimuw.bioputer.tav4sb.register.thread.Receiver;
import pl.edu.mimuw.bioputer.tav4sb.register.thread.Recorder;
import pl.edu.mimuw.bioputer.tav4sb.register.thread.Sender;
import pl.edu.mimuw.bioputer.tav4sb.register.utils.RegisterProperties;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.CommandLineArgs;

/**
 *
 * @author michal
 */
public class Main {

    private static final Logger logger = Logger.getLogger(Main.class);
    private static final RegisterProperties props = new RegisterProperties();

    private static Recorder getRecorder(CommandLineArgs args) throws IOException {
        String brokerUrl = args.getArgValue(props.RECORDER_BROKER_URL, props.getRecorderBrokerURL());
        String queueName = args.getArgValue(props.RECORDER_QUEUE_NAME, props.getRecorderQueueName());
        logger.debug("Recorder: brokerUrl="+brokerUrl+", queueName="+queueName+".");
        return new Recorder(brokerUrl, queueName);
    }

    private static Receiver getReceiver(CommandLineArgs args) throws IOException {
        String brokerUrl = args.getArgValue(props.RECEIVER_BROKER_URL, props.getReceiverBrokerURL());
        String queueName = args.getArgValue(props.RECEIVER_QUEUE_NAME, props.getReceiverQueueName());
        logger.debug("Receiver: brokerUrl="+brokerUrl+", queueName="+queueName+".");
        return new Receiver(brokerUrl, queueName);
    }

    private static Sender getSender(CommandLineArgs args) throws IOException {
        String hostName = args.getArgValue(props.SMTP_HOST_NAME, props.getSmtpHostName());
        String port = args.getArgValue(props.SMTP_PORT, props.getSmtpPort());
        String login = args.getArgValue(props.SMTP_LOGIN, props.getSmtpLogin());
        String password = args.getArgValue(props.SMTP_PASSWORD, props.getSmtpPassword());
        String starttls = args.getArgValue(props.SMTP_STARTTLS, props.getSmtpStarttls());
        String from = args.getArgValue(props.EMAIL_FROM, props.getEmailFrom());
        String signature = args.getArgValue(props.EMAIL_SIGNATURE, props.getEmailSignature());
        int sendPeriod = Integer.parseInt(args.getArgValue(props.EMAIL_SEND_PERIOD, props.getEmailSendPeriod()));
        return new Sender(hostName, port, login, password, starttls, from, signature, sendPeriod);
    }

    public static void main(String[] args) throws Exception {
        CommandLineArgs commadLine = new CommandLineArgs(args);

        Register.getInstance().addComputationFactory(new PrismComputationFactory());

        Thread[] threads = {
            new Thread(getRecorder(commadLine), "recorder"),
            new Thread(getReceiver(commadLine), "receiver"),
            new Thread(getSender(commadLine), "sender")
        };

        if (commadLine.isHelpSet()) {
            commadLine.printHelp(System.out);
            return;
        }

        logger.info("Starting threads");
        for (Thread t : threads) {
            t.start();
        }

        for (Thread t : threads) {
            try {
                t.join();
                logger.info(t.getName() + " done.");
            } catch (InterruptedException ex) {
                logger.error("interruption", ex);
            }
        }
    }
}
