/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.utils;

import java.io.File;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.mail.Address;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

/**
 *
 * @author michal
 */
public class MailSender {

    // private static final String SSL_FACTORY =
    // "javax.net.ssl.SSLSocketFactory";
    private static final String TEXT_PLAIN = "text/plain";
    private final String smtp_host;
    private final String smtp_port;
    private final String login;
    private final String password;
    private final String starttls;

    public MailSender(String host, String port, String login, String passwd,
            String starttls) {
        this.smtp_host = host;
        this.smtp_port = port;
        this.login = login;
        this.password = passwd;
        this.starttls = starttls;
    }

    public MailSender(String host, String port, String login, String passwd) {
        this(host, port, login, passwd, null);
    }

    private Session getSession() {
        Properties props = new Properties();
        props.setProperty("mail.transport.protocol", "smtp");
        props.setProperty("mail.host", smtp_host);
        props.put("mail.smtp.auth", "true");
        props.put("mail.smtp.port", smtp_port);
        if (starttls != null)
            props.put("mail.smtp.starttls.enable", starttls);
        // props.put("mail.smtp.socketFactory.port", smtp_port);
        // props.put("mail.smtp.socketFactory.class", SSL_FACTORY);
        // props.put("mail.smtp.socketFactory.fallback", "false");
        props.setProperty("mail.smtp.quitwait", "false");

        Session session = Session.getInstance(props,
                new javax.mail.Authenticator() {

                    @Override
                    protected PasswordAuthentication getPasswordAuthentication() {
                        return new PasswordAuthentication(login, password);
                    }
                });
        session.setDebug(true);
        return session;
    }

    public void send(String to, String from, String subject, String messageText,
            File[] fileAttachments) throws MessagingException {

        Message message = new MimeMessage(getSession());

        // Set up message meta
        Address[] fromAddr = { new InternetAddress(from) };
        message.addFrom(fromAddr);
        message.addRecipient(Message.RecipientType.TO, new InternetAddress(to));
        message.setSubject(subject);

        // Create the body and attachments parts
        Multipart multipart = new MimeMultipart();
        // #1: text body
        MimeBodyPart messageBodyPart = new MimeBodyPart();
        messageBodyPart.setText(messageText);
        // #2: attachments (if any)
        for (File fileAttachment : fileAttachments) {
            multipart.addBodyPart(messageBodyPart);
            messageBodyPart = new MimeBodyPart();
            DataSource source = new FileDataSource(fileAttachment);
            messageBodyPart.setDataHandler(new DataHandler(source));
            messageBodyPart.setFileName(fileAttachment.getName());
            multipart.addBodyPart(messageBodyPart);
        }
        // Put parts in message
        message.setContent(multipart);

        // Send message
        Transport.send(message);
    }

    public void send(String to, String from, String subject, String messageText)
            throws MessagingException {
        send(to, from, subject, messageText, new File[0]);
    }

}
