/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.utils.props;

/**
 *
 * @author trybik
 */
public abstract class PrefixedProperties extends PropertiesWrapper {
    private static final String PREFIX_SEPARATOR = "_";
    private final String prefix;

    public PrefixedProperties(String prefix) {
        this.prefix = prefix;
    }

    /**
     * Using this constructor is euqal to using directly the PropertiesWrapper.
     * It's here if both functionalities in one class are needed (i.e. prefixed
     * and normal properties getters).
     */
    public PrefixedProperties() {
        this(null);
    }

    public String resolvePropertyName(String propertyName) {
        return this.prefix != null ? this.prefix + PREFIX_SEPARATOR + propertyName : propertyName;
    }

    @Override
    protected String getProperty(String propertyName) {
        return super.getProperty(resolvePropertyName(propertyName));
    }
}
