/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.service.messages;

import java.util.LinkedList;
import java.util.List;
import javax.xml.bind.JAXBException;
import org.apache.log4j.Logger;
import pl.edu.mimuw.bioputer.tav4sb.producer.ServiceMessage;
import pl.edu.mimuw.bioputer.tav4sb.service.utils.WSProperties;
import pl.edu.mimuw.bioputer.tav4sb.utils.xml.XmlTranslator;

/**
 *
 * @author michal
 */
public abstract class WsMessage<WsParamT, CompParamT, CompResultT, WsResultT> extends ServiceMessage {

    private static final Logger logger = Logger.getLogger(WsMessage.class);
    public static final String context = "pl.edu.mimuw.bioputer.tav4sb.services.wsdl";
    public static final String RECEIVER = "RECEIVER";
    public static final String RECORDER = "RECORDER";
    private final XmlTranslator<CompParamT, CompResultT> translator;
    private List<String> results = new LinkedList<String>();
    private List<CompResultT> resultList = new LinkedList<CompResultT>();
    private final WsParamT param;
    private final WSProperties props;

    public WsMessage(WsParamT param, WSProperties props) {
        // TODO: XmlTranslator's singleton factory (registry); refactor as param
        this.translator = new XmlTranslator<CompParamT, CompResultT>(getContext());
        this.param = param;
        this.props = props;
    }

    /**
     * Return package context for the _inner_ parameters and results types.
     *
     * @return String with Java API package generated from WSDL included schema.
     */
    protected abstract String getContext();

    @Override
    protected String getQueueName() {
        return props.getQueueName();
    }

    @Override
    public String getMessageType() {
        return props.getMessageType();
    }

    @Override
    public String getBrokerUrl() {
        return props.getBrokerUrl();
    }

    @Override
    protected String getRecorderQueueName() {
        return RECORDER;
    }

    @Override
    protected String getReceiverQueueName() {
        return RECEIVER;
    }

    public List<String> getTextResults() {
        return results;
    }

    private XmlTranslator<CompParamT, CompResultT> getTranslator() {
        return this.translator;
    }

    protected abstract List<CompParamT> getObjectList();

    public List<String> paramsToString() throws JAXBException {
        logger.debug("begin");
        List<String> result = new LinkedList<String>();
        for (CompParamT o : getObjectList()) {
            String s = getTranslator().objectToString(o);
            logger.debug(s);
            result.add(s);
        }
        logger.debug("end");
        return result;
    }

    @Override
    protected List<String> getTextList() {
        try {
            return paramsToString();
        } catch (JAXBException e) {
            logger.error("parsing params error", e);
            return new LinkedList<String>();
        }
    }

    public CompResultT stringToResult(String s) throws JAXBException {
        return getTranslator().stringToObject(s);
    }

    @Override
    protected void addResult(String result) {
        logger.debug("begin");
        try {
            logger.debug(result);
            getTextResults().add(result);
            getResultList().add(stringToResult(result));
        } catch (JAXBException e) {
            logger.error("parsing result error", e);
            throw new IllegalArgumentException("parsing error");
        }
        logger.debug("end");
    }

    public List<CompResultT> getResultList() {
        return resultList;
    }

    public WsParamT getParam() {
        return this.param;
    }

    public abstract WsResultT getResult();
}
