/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.service;

import java.io.File;
import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import pl.edu.mimuw.bioputer.tav4sb.utils.file.FileReader;
import pl.edu.mimuw.bioputer.tav4sb.utils.xml.XmlTranslator;
import static org.junit.Assert.*;

/**
 *
 * @author michal
 */
public abstract class Tav4SBPortTypeImplAbstractTest {

    private static final Logger logger = Logger.getLogger(Tav4SBPortTypeImplAbstractTest.class);

    public Tav4SBPortTypeImplAbstractTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    protected abstract class Executor<A, B> {

        private final String path;

        public Executor(String path) {
            this.path = Tav4SBPortTypeImplAbstractTest.class.getClassLoader().getResource(path + ".xml").getPath();
            logger.debug("path: " + path);
        }

        protected abstract B doTest(A param, Tav4SBPortTypeImpl instance) throws Exception;

        public void test(XmlTranslator<B, A> translator) {
            logger.debug("begin");
            try {
                FileReader fr = new FileReader(new File(path));
                String input = fr.getContent();
                logger.debug("input:");
                logger.debug(input);
                A param = translator.stringToObject(input);
                Tav4SBPortTypeImpl instance = new Tav4SBPortTypeImpl();
                B result = doTest(param, instance);
                String strRes = translator.objectToString(result);
                logger.debug("result:");
                logger.debug(strRes);
            } catch (Exception e) {
                logger.error("exception", e);
                fail("exception: " + e.getMessage());
            }
            logger.debug("end");
        }

        public void test(String context) {
            test(new XmlTranslator<B, A>(context));
        }

        public void test(String paramContext, String resultContext) {
            test(new XmlTranslator<B, A>(resultContext, paramContext));
        }

        public void test(Class<A> paramClass, Class<B> resultClass) {
            test(paramClass.getPackage().getName(), resultClass.getPackage().getName());
        }

    }
}
